﻿/*
 * cs_rfm70.h
 *
 * Created: 29.10.2012 14:06:29
 *  Author: Christoph Söllner
 */ 

#include "config.h"
#include <avr/pgmspace.h>
#include <util/delay.h>

#ifndef CS_RFM70_H_
#define CS_RFM70_H_

// Define MAX/MIN macros
#ifndef max
#define max(a,b) ({ __typeof__ (a) _a = (a); __typeof__ (b) _b = (b); _a > _b ? _a : _b; })
#endif
#ifndef min
#define min(a,b) ({ __typeof__ (a) _a = (a); __typeof__ (b) _b = (b); _a < _b ? _a : _b; })
#endif

// Inline Defines for Chip-Enable and SPI Slave Select
#define rfm70_spi_slaveSelect()   PORTD&=0b11101111
#define rfm70_spi_slaveUnselect() PORTD|=0b00010000
#define rfm70_chipEnable()        PORTD|=0b00001000
#define rfm70_chipDisable()       PORTD&=0b11110111

// SPI(RFM70) commands
#define RFM70_CMD_READ_REG 0x00  // Define read command to register
#define RFM70_CMD_WRITE_REG 0x20 // Define write command to register
#define RFM70_CMD_READ_RX_PAYLOAD_WIDTH 0x60 // Define received payload width command
#define RFM70_CMD_READ_RX_PAYLOAD 0x61       // Define RX payload command
#define RFM70_CMD_WRITE_TX_PAYLOAD 0xA0       // Define TX payload command
#define RFM70_CMD_WRITE_TX_PAYLOAD_NOACK 0xB0 // Define TX payload NOACK command
#define RFM70_CMD_WRITE_ACK_WITH_PAYLOAD 0xa8 // Define command "ack with payload"
#define RFM70_CMD_FLUSH_TX 0xE1    // Define flush TX register command
#define RFM70_CMD_FLUSH_RX 0xE2    // Define flush RX register command
#define RFM70_CMD_REUSE_TX_PAYLOAD 0xE3 // Define reuse TX payload register command
#define RFM70_CMD_ACTIVATE 0x50  // Define feature activation command
#define RFM70_CMD_NOP_NOP 0xFF   // Define No Operation, might be used to read status register

// SPI(RFM70) registers(addresses)
#define RFM70_REG_CONFIG 0x00    // 'Config' register address
#define RFM70_REG_EN_AA 0x01     // 'Enable Auto Acknowledgment' register address
#define RFM70_REG_EN_RXADDR 0x02 // 'Enabled RX addresses' register address
#define RFM70_REG_SETUP_AW 0x03  // 'Setup address width' register address
#define RFM70_REG_SETUP_RETR 0x04 // 'Setup Auto. Retrans' register address
#define RFM70_REG_RF_CH 0x05      // 'RF channel' register address
#define RFM70_REG_RF_SETUP 0x06   // 'RF setup' register address
#define RFM70_REG_STATUS 0x07     // 'Status' register address
#define RFM70_REG_OBSERVE_TX 0x08 // 'Observe TX' register address
#define RFM70_REG_CD 0x09         // 'Carrier Detect' register address
#define RFM70_REG_RX_ADDR_P0 0x0A // 'RX address pipe0' register address
#define RFM70_REG_RX_ADDR_P1 0x0B // 'RX address pipe1' register address
#define RFM70_REG_RX_ADDR_P2 0x0C // 'RX address pipe2' register address
#define RFM70_REG_RX_ADDR_P3 0x0D // 'RX address pipe3' register address
#define RFM70_REG_RX_ADDR_P4 0x0E // 'RX address pipe4' register address
#define RFM70_REG_RX_ADDR_P5 0x0F // 'RX address pipe5' register address
#define RFM70_REG_TX_ADDR 0x10  // 'TX address' register address
#define RFM70_REG_RX_PW_P0 0x11 // 'RX payload width, pipe0' register address
#define RFM70_REG_RX_PW_P1 0x12 // 'RX payload width, pipe1' register address
#define RFM70_REG_RX_PW_P2 0x13 // 'RX payload width, pipe2' register address
#define RFM70_REG_RX_PW_P3 0x14 // 'RX payload width, pipe3' register address
#define RFM70_REG_RX_PW_P4 0x15 // 'RX payload width, pipe4' register address
#define RFM70_REG_RX_PW_P5 0x16 // 'RX payload width, pipe5' register address
#define RFM70_REG_FIFO_STATUS 0x17 // 'FIFO Status Register' register address
#define RFM70_REG_DYNPD 0x1c       // 'Enable dynamic payload length' register address
#define RFM70_REG_FEATURE 0x1d     // 'Feature' register address

//interrupt status
#define RFM70_IRQ_STATUS_RX_DR   0x40 // Status bit RX_DR IRQ
#define RFM70_IRQ_STATUS_TX_DS   0x20 // Status bit TX_DS IRQ
#define RFM70_IRQ_STATUS_MAX_RT  0x10 // Status bit MAX_RT IRQ
#define RFM70_IRQ_STATUS_TX_FULL 0x01

// FIFO_STATUS definitions
#define RFM70_FIFO_STATUS_RX_EMPTY  0b00000001 // RX-FIFO empty?
#define RFM70_FIFO_STATUS_RX_FULL   0b00000010 // RX-FIFO full, =3 packets waiting to be read
#define RFM70_FIFO_STATUS_TX_EMPTY  0b00010000 // TX-FIFO empty
#define RFM70_FIFO_STATUS_TX_FULL   0b00100000 // TX-FIFO full, forgot to enable CE=1 or bad link with retransmit enabled?
#define RFM70_FIFO_STATUS_TX_REUSE  0b01000000 // TX unit continuously transmits a paket. Stop with TX_FLUSH or WRITE_TX_PAYLOAD



// ENUMS
enum RFM70_BANKSELECT { RFM70_SELECT_BANK_0, RFM70_SELECT_BANK_1 };
enum RFM70_TXPOWERSELECT { RFM70_RF_PWR_MINUS_10_DBM, RFM70_RF_PWR_MINUS_5_DBM, RFM70_RF_PWR_0_DBM, RFM70_RF_PWR_PLUS_5_DBM };

// FUNCTIONS
unsigned char rfm70_spi_rw(unsigned char value);
void rfm70_writeSingleByteToRegister(unsigned char destinationRegister, unsigned char value);
unsigned char rfm70_readSingleByteFromRegister(unsigned char destinationRegister);
void rfm70_readIntoBufferFromRegisterN(unsigned char rfm70DestinationRegister, unsigned char *destinationBuffer, unsigned char destinationBufferLength);
void rfm70_writeToRegisterFromBufferN(unsigned char rfm70DestinationRegister, unsigned char *sourceBuffer, unsigned char sourceBufferLength);
void rfm70_selectReceiveMode();
void rfm70_selectTransmitMode();
void rfm70_selectBank(char _cfg);
void rfm70_selectChannel(unsigned char ch);
unsigned char rfm70_sendData(unsigned char *payloadPointer, size_t payloadSize);
void rfm70_initChipRegisters();
void rfm70_initPortsAndSpi();
unsigned char rfm70_getPipeNumberForFifoData();
unsigned char rfm70_readFromFifoIntoBuffer(unsigned char *destinationBuffer, unsigned char maxReadCharacters);
void rfm70_setPipeAddress(unsigned char targetPipe, unsigned char *addressValue, unsigned char addressLength);









#endif /* CS_RFM70_H_ */